package java.util.zip;

import java.util.concurrent.TimeUnit;
import java.util.Objects;
import java.nio.file.attribute.FileTime;

public class ZipEntry implements ZipConstants, Cloneable
{
    String name;
    long xdostime;
    FileTime mtime;
    FileTime atime;
    FileTime ctime;
    long crc;
    long size;
    long csize;
    int method;
    int flag;
    byte[] extra;
    String comment;
    public static final int STORED = 0;
    public static final int DEFLATED = 8;
    static final long DOSTIME_BEFORE_1980 = 2162688L;
    private static final long UPPER_DOSTIME_BOUND = 4036608000000L;
    
    public ZipEntry(String s) {
        this.xdostime = -1L;
        this.crc = -1L;
        this.size = -1L;
        this.csize = -1L;
        this.method = -1;
        this.flag = 0;
        Objects.requireNonNull(s, "name");
        if (s.length() > 65535) {
            throw new IllegalArgumentException("entry name too long");
        }
        this.name = s;
        return;
    }
    
    public ZipEntry(ZipEntry obj) {
        this.xdostime = -1L;
        this.crc = -1L;
        this.size = -1L;
        this.csize = -1L;
        this.method = -1;
        this.flag = 0;
        Objects.requireNonNull(obj, "entry");
        this.name = obj.name;
        this.xdostime = obj.xdostime;
        this.mtime = obj.mtime;
        this.atime = obj.atime;
        this.ctime = obj.ctime;
        this.crc = obj.crc;
        this.size = obj.size;
        this.csize = obj.csize;
        this.method = obj.method;
        this.flag = obj.flag;
        this.extra = obj.extra;
        this.comment = obj.comment;
    }
    
    ZipEntry() {
        this.xdostime = -1L;
        this.crc = -1L;
        this.size = -1L;
        this.csize = -1L;
        this.method = -1;
        this.flag = 0;
    }
    
    public String getName() {
        return this.name;
    }
    
    public void setTime(long value) {
        this.xdostime = ZipUtils.javaToExtendedDosTime(value);
        if (this.xdostime != 2162688L && value <= 4036608000000L) {
            this.mtime = null;
        }
        else {
            this.mtime = FileTime.from(value, TimeUnit.MILLISECONDS);
        }
    }
    
    public long getTime() {
        if (this.mtime != null) {
            return this.mtime.toMillis();
        }
        return (this.xdostime != -1L) ? ZipUtils.extendedDosToJavaTime(this.xdostime) : -1L;
    }
    
    public ZipEntry setLastModifiedTime(FileTime obj) {
        this.mtime = (FileTime)Objects.requireNonNull(obj, "lastModifiedTime");
        this.xdostime = ZipUtils.javaToExtendedDosTime(obj.to(TimeUnit.MILLISECONDS));
        return this;
    }
    
    public FileTime getLastModifiedTime() {
        if (this.mtime != null) {
            return this.mtime;
        }
        if (this.xdostime == -1L) {
            return null;
        }
        return FileTime.from(this.getTime(), TimeUnit.MILLISECONDS);
    }
    
    public ZipEntry setLastAccessTime(FileTime obj) {
        this.atime = (FileTime)Objects.requireNonNull(obj, "lastAccessTime");
        return this;
    }
    
    public FileTime getLastAccessTime() {
        return this.atime;
    }
    
    public ZipEntry setCreationTime(FileTime obj) {
        this.ctime = (FileTime)Objects.requireNonNull(obj, "creationTime");
        return this;
    }
    
    public FileTime getCreationTime() {
        return this.ctime;
    }
    
    public void setSize(long size) {
        if (size < 0L) {
            throw new IllegalArgumentException("invalid entry size");
        }
        this.size = size;
        return;
    }
    
    public long getSize() {
        return this.size;
    }
    
    public long getCompressedSize() {
        return this.csize;
    }
    
    public void setCompressedSize(long csize) {
        this.csize = csize;
    }
    
    public void setCrc(long crc) {
        if (crc < 0L || crc > 4294967295L) {
            throw new IllegalArgumentException("invalid entry crc-32");
        }
        this.crc = crc;
        return;
    }
    
    public long getCrc() {
        return this.crc;
    }
    
    public void setMethod(int method) {
        if (method != 0 && method != 8) {
            throw new IllegalArgumentException("invalid compression method");
        }
        this.method = method;
        return;
    }
    
    public int getMethod() {
        return this.method;
    }
    
    public void setExtra(byte[] array) {
        this.setExtra0(array, false);
    }
    
    void setExtra0(byte[] extra, boolean b) {
        if (extra != null) {
            if (extra.length > 65535) {
                throw new IllegalArgumentException("invalid extra field length");
            }
            int n = 0;
            int length = extra.length;
            while (n + 4 < length) {
                int get16 = ZipUtils.get16(extra, n);
                int get17 = ZipUtils.get16(extra, n + 2);
                n += 4;
                if (n + get17 > length) {
                    break;
                }
                switch (get16) {
                    case 1: {
                        if (b && get17 >= 16) {
                            this.size = ZipUtils.get64(extra, n);
                            this.csize = ZipUtils.get64(extra, n + 8);
                            break;
                        }
                        break;
                    }
                    case 10: {
                        if (get17 < 32) {
                            break;
                        }
                        int n2 = n + 4;
                        if (ZipUtils.get16(extra, n2) == 1) {
                            if (ZipUtils.get16(extra, n2 + 2) != 24) {
                                break;
                            }
                            this.mtime = ZipUtils.winTimeToFileTime(ZipUtils.get64(extra, n2 + 4));
                            this.atime = ZipUtils.winTimeToFileTime(ZipUtils.get64(extra, n2 + 12));
                            this.ctime = ZipUtils.winTimeToFileTime(ZipUtils.get64(extra, n2 + 20));
                            break;
                        }
                        else {
                            break;
                        }
                    }
                    case 21589: {
                        int unsignedInt = Byte.toUnsignedInt(extra[n]);
                        int n3 = 1;
                        if ((unsignedInt & 0x1) != 0x0 && n3 + 4 <= get17) {
                            this.mtime = ZipUtils.unixTimeToFileTime(ZipUtils.get32(extra, n + n3));
                            n3 += 4;
                        }
                        if ((unsignedInt & 0x2) != 0x0 && n3 + 4 <= get17) {
                            this.atime = ZipUtils.unixTimeToFileTime(ZipUtils.get32(extra, n + n3));
                            n3 += 4;
                        }
                        if ((unsignedInt & 0x4) != 0x0 && n3 + 4 <= get17) {
                            this.ctime = ZipUtils.unixTimeToFileTime(ZipUtils.get32(extra, n + n3));
                            n3 += 4;
                            break;
                        }
                        break;
                    }
                }
                n += get17;
            }
        }
        this.extra = extra;
    }
    
    public byte[] getExtra() {
        return this.extra;
    }
    
    public void setComment(String comment) {
        this.comment = comment;
    }
    
    public String getComment() {
        return this.comment;
    }
    
    public boolean isDirectory() {
        return this.name.endsWith("/");
    }
    
    @Override
    public String toString() {
        return this.getName();
    }
    
    @Override
    public int hashCode() {
        return this.name.hashCode();
    }
    
    public Object clone() {
        try {
            ZipEntry zipEntry = (ZipEntry)super.clone();
            zipEntry.extra = ((this.extra == null) ? null : ((byte[])this.extra.clone()));
            return zipEntry;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
}
